"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatMessages = exports.ChatMessageSkeleton = void 0;
const react_1 = require("@chakra-ui/react");
const react_2 = __importStar(require("react"));
const throttle_1 = __importDefault(require("lodash/throttle"));
const message_1 = require("./message");
const MyFlex_1 = require("./MyFlex");
const react_async_hook_1 = require("react-async-hook");
const spl_utils_1 = require("@strata-foundation/spl-utils");
const INACTIVE_TIME = 60; // After 1 minute, new grouping
const INFINITE_SCROLL_THRESHOLD = 300;
const FETCH_COUNT = 50;
const ChatMessageSkeleton = () => (react_2.default.createElement(MyFlex_1.Flex, { padding: 2, gap: 2 },
    react_2.default.createElement(MyFlex_1.Flex, null,
        react_2.default.createElement(react_1.SkeletonCircle, { size: "9" })),
    react_2.default.createElement(MyFlex_1.Flex, { grow: 1, direction: "column", gap: 2 },
        react_2.default.createElement(MyFlex_1.Flex, { gap: 2 },
            react_2.default.createElement(react_1.Skeleton, { height: "8px", w: "120px" }),
            react_2.default.createElement(react_1.Skeleton, { height: "8px", w: "60px" })),
        react_2.default.createElement(react_1.Skeleton, { height: "8px", w: "full" }),
        react_2.default.createElement(react_1.Skeleton, { height: "8px", w: "full" }))));
exports.ChatMessageSkeleton = ChatMessageSkeleton;
const ChatMessages = ({ isLoading, isLoadingMore, hasMore, fetchMore = () => null, scrollRef, messages = [], }) => {
    const myScrollRef = (0, react_2.useRef)(null);
    if (!scrollRef)
        scrollRef = myScrollRef;
    // On render if we dont have a scroll bar
    // and we have hasMore then fetch initialMore
    (0, react_2.useEffect)(() => {
        if (scrollRef.current.scrollHeight == scrollRef.current.offsetHeight &&
            hasMore &&
            !isLoading) {
            fetchMore(FETCH_COUNT);
        }
    }, [scrollRef, hasMore, isLoading, fetchMore]);
    const handleOnScroll = (0, react_2.useCallback)((0, throttle_1.default)((e) => {
        const scrollOffset = e.target.scrollHeight + e.target.scrollTop;
        if (scrollOffset <= e.target.offsetHeight + INFINITE_SCROLL_THRESHOLD &&
            !isLoadingMore &&
            hasMore) {
            fetchMore(FETCH_COUNT);
        }
    }, 300), [isLoadingMore, fetchMore, hasMore]);
    const loaders = (0, react_2.useMemo)(() => {
        return !messages.length ? (Array.from(Array(FETCH_COUNT).keys()).map((x, index) => (react_2.default.createElement(exports.ChatMessageSkeleton, { key: `skeleton-${index}` })))) : (react_2.default.createElement(exports.ChatMessageSkeleton, null));
    }, [messages.length]);
    const { execute: scrollToMessage } = (0, react_async_hook_1.useAsyncCallback)(function (id) {
        return __awaiter(this, void 0, void 0, function* () {
            // listen to escape key press to break loop
            let breakLoop = false;
            function keyPress(e) {
                if (e.key === "Escape") {
                    breakLoop = true;
                }
            }
            document.addEventListener("keypress", keyPress);
            while (!breakLoop && hasMore && scrollRef.current) {
                let findElem = document.getElementById(id);
                if (findElem) {
                    findElem.scrollIntoView({ behavior: "smooth", block: "center" });
                    return;
                }
                // scroll to the top which should load more messages
                scrollRef.current.scroll({
                    top: -scrollRef.current.scrollHeight,
                    behavior: "smooth",
                });
                yield (0, spl_utils_1.sleep)(300);
            }
            document.removeEventListener("keypress", keyPress);
        });
    });
    return (react_2.default.createElement(MyFlex_1.Flex, { grow: 1, overflowY: "auto", direction: "column-reverse", ref: scrollRef, onScroll: handleOnScroll }, messages === null || messages === void 0 ? void 0 :
        messages.map((msg, index) => (react_2.default.createElement(message_1.MemodMessage, Object.assign({ scrollToMessage: scrollToMessage, key: msg === null || msg === void 0 ? void 0 : msg.id }, msg, { showUser: !(messages[index + 1] &&
                messages[index + 1].sender.equals(msg.sender) &&
                messages[index + 1].endBlockTime >=
                    (msg.startBlockTime || new Date().valueOf() / 1000) -
                        INACTIVE_TIME) || !!msg.reply })))),
        (isLoading || isLoadingMore) && loaders));
};
exports.ChatMessages = ChatMessages;
//# sourceMappingURL=ChatMessages.js.map